package hero.client.manager;

/*
*
* Manager.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.util.BonitaClient;
import hero.util.JMSServices;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;
import java.util.Collection;
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

public class Manager extends JPanel {

	String url = null, host = null;
	int port; 
	String userName = null;
	String userPassword = null;
	Collection projects = new ArrayList();
	JProjectList projectList = null;
	final JFrame p = new JFrame();
	BonitaClient soapclient;

	public Manager(String userName, String userPassword) throws Exception {

		this.userName = userName;
		this.userPassword = userPassword;
  
        // Obtains bonita host and port
		url = System.getProperty("bonita.host");

	    host = url.substring(7);
		int point = host.indexOf(":");
		url = host;
		this.host = host.substring(0,point);
		int portPoint = url.indexOf("/");
		if (portPoint != -1) 
			this.port = new Integer(url.substring(point+1,portPoint)).intValue(); // ant application
		else
			this.port = new Integer(url.substring(point+1)).intValue(); //Java Web Start application

		BonitaClient soapclient = new BonitaClient(host, port, "/bonita_ws/services/");
		soapclient.login(userName, userPassword);
		this.soapclient = soapclient;
		buildView();
	}

	public void buildView() throws Exception {
		String subscription = "(event = 'addUserProject' AND userName = '" + this.userName + "')" + " OR";

		this.projects = soapclient.getProjects();
		Iterator i = projects.iterator();
		while (i.hasNext())
			subscription = subscription + " projectName = '" + (String) i.next() + "' OR";

		MListener ml = MListener.getInstance(this.userName, this.userPassword, this.host, this.port);
		ml.subscription(subscription.substring(0, subscription.length() - 3));
		projectList = new JProjectList(this.soapclient, ml, this.projects);
		JTodoList todoList = new JTodoList(this.soapclient, ml);
		JActivityList activityList = new JActivityList(this.soapclient, ml);

		projectList.addPropertyChangeListener(JProjectList.SELECTED, todoList);
		projectList.addPropertyChangeListener(JProjectList.SELECTED, activityList);

		hero.client.manager.Menubar menubar = new hero.client.manager.Menubar(this);
		p.setJMenuBar(menubar);
		Container c = p.getContentPane();
		c.setLayout(new GridLayout(3, 1));
		c.add(projectList);
		c.add(todoList);
		c.add(activityList);

		p.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				try {
					JMSServices jms = JMSServices.getInstance();
					jms.closeConnection();
					System.exit(0);
				} catch (Exception ex) {
				}
			}
		});

		p.setSize(new Dimension(150, 400));

		p.setTitle("Activity Manager - BnUser: " + userName);
		p.pack();
		p.show();
	}

	public void CloneProject() {
		JPanel jp = new JPanel();

		Object[] possibleValues = this.projects.toArray();
		JComboBox projectNames = new JComboBox(possibleValues);
		jp.setLayout(new GridLayout(0, 2));
		jp.add(new JLabel("Choose BnProject ", SwingConstants.LEFT));
		jp.add(projectNames);

		JTextField newProject = new JTextField("", 0);
		jp.setLayout(new GridLayout(0, 2));
		jp.add(new JLabel("BnProject Name ", SwingConstants.LEFT));
		jp.add(newProject);

		int cloneOption = JOptionPane.showConfirmDialog(null, jp, "Clone BnProject", JOptionPane.OK_CANCEL_OPTION);
		if (cloneOption != JOptionPane.CANCEL_OPTION) {
			while ((newProject.getText()).equals("") && cloneOption != JOptionPane.CANCEL_OPTION) {
				JOptionPane.showMessageDialog(null, "Please enter project name... ");
				cloneOption = JOptionPane.showConfirmDialog(null, jp, "Clone BnProject", JOptionPane.OK_CANCEL_OPTION);
			}
			if (cloneOption != JOptionPane.CANCEL_OPTION) {
				try {
					if (exists(newProject.getText())) {
						JOptionPane.showMessageDialog(null, "BnProject name already exist... ");
					} else{
						soapclient.cloneProject((String) projectNames.getSelectedItem(), newProject.getText());
						this.projects = soapclient.getProjects();
					}
				} catch (Exception e) {
					e.printStackTrace();
					JOptionPane.showMessageDialog(null, "BnProject name already exist... ");
				}
			}
		}
	}

	private boolean exists(String project) {
		try {
			Object[] elems = this.projects.toArray();
			for (int i = 0; i < elems.length; i++) {
				if (((String) elems[i]).equalsIgnoreCase(project))
					return true;
			}
			return false;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return true;
	}

	public void NewProject() {
		projectList.newProject();
	}

	static public void main(String[] args) {
		boolean exit = false;
		String userName = null;
		String userPassword = null;

		if (System.getProperty("user.login") != null && System.getProperty("user.password") != null) {
			try {
				userName = System.getProperty("user.login");
				char[] password = System.getProperty("user.password").toCharArray();
				userPassword = new String(password);
				new Manager(userName, userPassword);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, "Login Manager failed... you cannot acces to system ");
				System.exit(0);
			}
		} else {
			while (!exit) {
				JPanel jp = new JPanel();

				JTextField login = new JTextField("", 0);
				jp.setLayout(new GridLayout(0, 2));
				jp.add(new JLabel("Login: ", SwingConstants.LEFT));
				jp.add(login);

				JPasswordField passwd = new JPasswordField("", 0);
				jp.setLayout(new GridLayout(0, 2));
				jp.add(new JLabel("Password: ", SwingConstants.LEFT));
				jp.add(passwd);

				try {
					if (System.getProperty("user.login") != null && System.getProperty("user.password") != null) {
						userName = System.getProperty("user.login");
						char[] password = System.getProperty("user.password").toCharArray();
						userPassword = new String(password);
						new Manager(userName, userPassword);
						exit = true;
					} else {
						int option = JOptionPane.showConfirmDialog(null, jp, "Manager Login", JOptionPane.OK_CANCEL_OPTION);
						if (option != JOptionPane.CANCEL_OPTION) {
							while (((login.getText()).equals("") || (passwd.getPassword()).length == 0) && option != JOptionPane.CANCEL_OPTION) {
								JOptionPane.showMessageDialog(null, "Please enter login and password values... ");
								option = JOptionPane.showConfirmDialog(null, jp, "Manager Login", JOptionPane.OK_CANCEL_OPTION);
							}
							if (option != JOptionPane.CANCEL_OPTION) {
								userName = (String) login.getText();
								char[] password = passwd.getPassword();
								userPassword = new String(password);
								new Manager(userName, userPassword);
								exit = true;
							} else
								System.exit(0);
						} else
							System.exit(0);
					}
				} catch (Exception e) {
					e.printStackTrace();
					JOptionPane.showMessageDialog(null, "Login Manager failed... you cannot access to system ");
					exit = false;
				}
			}
		}
	}
}


