package hero.client.test;

/*
*
* EngineTest.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectValue;
import hero.interfaces.Constants;
import hero.interfaces.Engine;
import hero.interfaces.EngineHome;
import hero.interfaces.EngineUtil;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.util.EngineException;
import hero.util.HeroException;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class EngineTests extends TestCase {
	private ProjectSessionHome psh;
	private ProjectSession ps;
	private EngineHome eh;
	private Engine eng;

	public EngineTests(String testname) {
		super(testname);
	}

	public static TestSuite suite() {
		return new TestSuite(EngineTests.class);
	}

	public void setUp() throws Exception {
		psh = ProjectSessionUtil.getHome();
		eh = EngineUtil.getHome();
	}

	public void testEngineAND() throws Exception {
		ProjectSession proj;
		BnProjectValue pm;
		BnNodeValue nd;
		BnEdgeValue ev;

		// creation du projet		
		proj = psh.create();
		proj.initProject("work");
		proj.addNode("init", Constants.Nd.AND_JOIN_NODE);
		proj.addNode("sp1", Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		proj.addNode("ac1", Constants.Nd.AND_JOIN_NODE);
		proj.addNode("ac2", Constants.Nd.AND_JOIN_NODE);
		proj.addNode("jn1", Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		proj.addNode("ac3", Constants.Nd.AND_JOIN_NODE);
		String e1 = proj.addEdge("init", "sp1");
		String e2 = proj.addEdge("sp1", "ac1");
		String e3 = proj.addEdge("sp1", "ac2");
		String e4 = proj.addEdge("ac1", "jn1");
		String e5 = proj.addEdge("ac2", "jn1");
		String e6 = proj.addEdge("jn1", "ac3");
		proj.addRole("test", "testeur");
		proj.setUserRole("admin", "test");

		eng = eh.create();
		eng.initProject("work");

		try {
			eng.startActivity("ac1");
			assertTrue("start ac1 should have failed", false);
		} catch (EngineException e) {
			// normal behavior
		}

		try {
			eng.terminateActivity("init");
			assertTrue("terminate init should have failed", false);
		} catch (EngineException e) {
			// normal behavior
		}

		eng.startActivity("init");		
		proj = psh.create();
		proj.initProject("work");

		assertTrue("BnNode State Error init",proj.getNodeValue("init").getState() == Constants.Nd.EXECUTING);
		assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode State Error sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.ANTICIPATING);
		assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode State Error ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("BnEdge State Error e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode State Error ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("BnEdge State Error e4",proj.getEdgeValue(e4).getState() == Constants.Ed.INITIAL);
		assertTrue("BnEdge State Error e5",	proj.getEdgeValue(e5).getState() == Constants.Ed.INITIAL);
		assertTrue("BnNode State Error jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.INITIAL);
		assertTrue(	"BnEdge State Error e6",proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
		assertTrue("BnNode State Error ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);

		try {
			eng.terminateActivity("sp1");
			assertTrue("terminate sp1 should have failed", false);
		} catch (EngineException e) {
			// normal behavior
		}

		eng.terminateActivity("init");

		assertTrue("BnNode State Error Term init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge State Error Term e1",	proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error Term sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.READY);
		assertTrue("BnEdge State Error e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.READY);
		assertTrue("BnEdge State Error e4",proj.getEdgeValue(e4).getState() == Constants.Ed.INITIAL);
		assertTrue("BnEdge State Error e5",proj.getEdgeValue(e5).getState() == Constants.Ed.INITIAL);
		assertTrue("BnNode State Error jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.INITIAL);
		assertTrue("BnEdge State Error e6",proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
		assertTrue("BnNode State Error ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);

		eng.startActivity("ac1");

		assertTrue("BnNode State Error start ac1 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge State Error start ac1 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error start ac1 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge State Error start ac1 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error start ac1 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.EXECUTING);
		assertTrue("BnEdge State Error start ac1 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error start ac1 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.READY);
		assertTrue("BnEdge State Error start ac1 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnEdge State Error start ac1 : e5",proj.getEdgeValue(e5).getState() == Constants.Ed.INITIAL);
		assertTrue("BnNode State Error start ac1 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.INITIAL);
		assertTrue("BnEdge State Error start ac1 : e6",proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
		assertTrue("BnNode State Error start ac1 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);

		eng.startActivity("ac2");

		assertTrue("BnNode State Error start ac2 : init",proj.getNodeValue("init").getState()== Constants.Nd.TERMINATED);
		assertTrue("BnEdge State Error start ac2 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error start ac2 : sp1",proj.getNodeValue("sp1").getState()== Constants.Nd.TERMINATED);
		assertTrue("BnEdge State Error start ac2 : e2",	proj.getEdgeValue(e2).getState()== Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error start ac2 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.EXECUTING);
		assertTrue("BnEdge State Error start ac2 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error start ac2 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.EXECUTING);
		assertTrue("BnEdge State Error start ac2 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnEdge State Error start ac2 : e5",proj.getEdgeValue(e5).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode State Error start ac2 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.ANTICIPATING);
		assertTrue("BnEdge State Error start ac2 : e6",	proj.getEdgeValue(e6).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode State Error start ac2 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.ANTICIPABLE);

		eng.startActivity("ac3");

		try {
			eng.terminate();
			assertTrue("should have failed", false);
		} catch (HeroException h) {
			// normal behavior
		}

		assertTrue("BnNode State Error start ac3 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge State Error start ac3 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode State Error start ac3 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge State Error start ac3 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error start ac3 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.EXECUTING);
		assertTrue("BnEdge Value Error start ac3 : e3",	proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error start ac3 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.EXECUTING);
		assertTrue("BnEdge Value Error start ac3 : e4",	proj.getEdgeValue(e4).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnEdge Value Error start ac3 : e5",	proj.getEdgeValue(e5).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode Value Error start ac3 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.ANTICIPATING);
		assertTrue("BnEdge Value Error start ac3 : e6",	proj.getEdgeValue(e6).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode Value Error start ac3 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.ANTICIPATING);

		try {
			eng.terminateActivity("ac3");
			assertTrue("terminate ac3 should have failed", false);
		} catch (EngineException e) {
			// normal behavior
		}

		eng.terminateActivity("ac1");

		assertTrue("BnNode Value Error terminate ac1 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac1 : e1",	proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac1 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac1 : e2",	proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac1 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac1 : e3",	proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac1 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.EXECUTING);
		assertTrue("BnEdge Value Error terminate ac1 : e4",	proj.getEdgeValue(e4).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnEdge Value Error terminate ac1 : e5", proj.getEdgeValue(e5).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode Value Error terminate ac1 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.ANTICIPATING);
		assertTrue("BnEdge Value Error terminate ac1 : e6",	proj.getEdgeValue(e6).getState() == Constants.Ed.ANTICIPATING);
		assertTrue("BnNode Value Error terminate ac1 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.ANTICIPATING);

		eng.terminateActivity("ac2");

		assertTrue("BnNode Value Error terminate ac2 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e1",	proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e2",	proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e3",	proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnEdge Value Error terminate ac2 : e5",	proj.getEdgeValue(e5).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e6",	proj.getEdgeValue(e6).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.EXECUTING);

		eng.terminateActivity("ac3");

		assertTrue("BnNode Value Error terminate ac2 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : sp1",	proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e4",	proj.getEdgeValue(e4).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnEdge Value Error terminate ac2 : e5",	proj.getEdgeValue(e5).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.TERMINATED);
		assertTrue("BnEdge Value Error terminate ac2 : e6",	proj.getEdgeValue(e6).getState() == Constants.Ed.ACTIVE);
		assertTrue("BnNode Value Error terminate ac2 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.TERMINATED);

		try {
			eng.startActivity("ac2");
			assertTrue("start ac2 should have failed", false);
		} catch (EngineException e) {
			// normal behavior
		}

		eng.terminate();
		assertTrue("Wrong project state",proj.getDetails().getState() == Constants.Pj.TERMINATED);
	}


	public void testEngineOR() throws Exception {
	    ProjectSession proj;
	    BnProjectValue pm;
	    BnNodeValue nd;
	    BnEdgeValue ev;
	    
	    // creation du projet
	    proj = psh.create();
		proj.initProject("workOR");
	    proj.addNode("init", Constants.Nd.AND_JOIN_NODE);
	    proj.addNode("sp1", Constants.Nd.AND_JOIN_NODE);
	    proj.addNode("ac1", Constants.Nd.AND_JOIN_NODE);
	    proj.addNode("ac2", Constants.Nd.AND_JOIN_NODE);
	    proj.addNode("jn1", Constants.Nd.OR_JOIN_NODE);
	    proj.addNode("ac3", Constants.Nd.AND_JOIN_NODE);
	    String e1 = proj.addEdge("init", "sp1");
	    String e2 = proj.addEdge("sp1", "ac1");
	    String e3 = proj.addEdge("sp1", "ac2");
	    String e4 = proj.addEdge("ac1", "jn1");
	    String e5 = proj.addEdge("ac2", "jn1");
	    String e6 = proj.addEdge("jn1", "ac3");
	    proj.addRole("test", "testeur");
	    proj.setUserRole("admin", "test");
	    
	    eng = eh.create();
	    eng.initProject("workOR");
	    eng.startActivity("init");
	    eng.startActivity("sp1");
		proj = psh.create();
		proj.initProject("workOR");
	    eng.terminateActivity("init");
	    
	    assertTrue("BnNode State Error init",	proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.EXECUTING);
	    assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.ANTICIPABLE);
	    assertTrue("BnEdge State Error e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.ANTICIPABLE);
	    assertTrue("BnEdge State Error e4",proj.getEdgeValue(e4).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnEdge State Error e5",	proj.getEdgeValue(e5).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnNode State Error jn1",	proj.getNodeValue("jn1").getState() == Constants.Nd.INITIAL);
	    assertTrue(	"BnEdge State Error e6",	proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnNode State Error ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);
		
	    eng.startActivity("ac1");
		
	    assertTrue("BnNode State Error start ac1 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.EXECUTING);
	    assertTrue("BnEdge State Error start ac1 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error start ac1 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.ANTICIPATING);
	    assertTrue("BnEdge State Error start ac1 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error start ac1 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.ANTICIPABLE);
	    assertTrue("BnEdge State Error start ac1 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnEdge State Error start ac1 : e5",proj.getEdgeValue(e5).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnNode State Error start ac1 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.ANTICIPABLE);
	    assertTrue("BnEdge State Error start ac1 : e6",proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnNode State Error start ac1 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);

	    eng.startActivity("ac2");
	    
	    assertTrue("BnNode State Error start ac1 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.EXECUTING);
	    assertTrue("BnEdge State Error start ac1 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error start ac1 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.ANTICIPATING);
	    assertTrue("BnEdge State Error start ac1 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error start ac1 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.ANTICIPATING);
	    assertTrue("BnEdge State Error start ac1 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnEdge State Error start ac1 : e5",proj.getEdgeValue(e5).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error start ac1 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.ANTICIPABLE);
	    assertTrue("BnEdge State Error start ac1 : e6",proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnNode State Error start ac1 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);
	    
	    eng.terminateActivity("sp1");

	    assertTrue("BnNode State Error start ac1 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.EXECUTING);
	    assertTrue("BnEdge State Error start ac1 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.EXECUTING);
	    assertTrue("BnEdge State Error start ac1 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnEdge State Error start ac1 : e5",proj.getEdgeValue(e5).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error start ac1 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.ANTICIPABLE);
	    assertTrue("BnEdge State Error start ac1 : e6",proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnNode State Error start ac1 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);
	    
	    eng.terminateActivity("ac1");
	    
	    assertTrue("BnNode State Error start ac1 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.EXECUTING);
	    assertTrue("BnEdge State Error start ac1 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnEdge State Error start ac1 : e5",proj.getEdgeValue(e5).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error start ac1 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.READY);
	    assertTrue("BnEdge State Error start ac1 : e6",proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnNode State Error start ac1 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);
	    
	    eng.terminateActivity("ac2");
	    
	    assertTrue("BnNode State Error start ac1 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnEdge State Error start ac1 : e5",proj.getEdgeValue(e5).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.READY);
	    assertTrue("BnEdge State Error start ac1 : e6",proj.getEdgeValue(e6).getState() == Constants.Ed.INITIAL);
	    assertTrue("BnNode State Error start ac1 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.INITIAL);
	    
	    eng.startActivity("jn1");
	    eng.startActivity("ac3");
	    
	    assertTrue("BnNode State Error start ac1 : init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.TERMINATED);
	    assertTrue("BnEdge State Error start ac1 : e4",proj.getEdgeValue(e4).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnEdge State Error start ac1 : e5",proj.getEdgeValue(e5).getState() == Constants.Ed.ACTIVE);
	    assertTrue("BnNode State Error start ac1 : jn1",proj.getNodeValue("jn1").getState() == Constants.Nd.EXECUTING);
	    assertTrue("BnEdge State Error start ac1 : e6",proj.getEdgeValue(e6).getState() == Constants.Ed.ANTICIPATING);
	    assertTrue("BnNode State Error start ac1 : ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.ANTICIPATING);
	    
	    try {
		eng.terminateActivity("ac3");
		assertTrue("should have failed", false);
	    } catch (HeroException h) {
		// normal behavior
	    }
	    
	    eng.terminateActivity("jn1");
	    eng.terminateActivity("ac3");
	    
	    eng.terminate();
	    assertTrue("Wrong project state",	proj.getDetails().getState() == Constants.Pj.TERMINATED);						
	}
    
    public void testCancelANDActivity() throws Exception {
	ProjectSession proj;
	BnProjectValue pm;
	BnNodeValue nd;
	BnEdgeValue ev;
	
	// creation du projet	
	proj = psh.create();
	proj.initProject("workCancel");
	proj.addNode("init", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("sp1", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("ac1", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("ac2", Constants.Nd.AND_JOIN_NODE);

	String e1 = proj.addEdge("init", "sp1");
	String e2 = proj.addEdge("sp1", "ac1");
	String e3 = proj.addEdge("sp1", "ac2");
	
	proj.addRole("test", "testeur");
	proj.setUserRole("admin", "test");
	
	eng = eh.create();
	eng.initProject("workCancel");
	eng.startActivity("init");
	eng.startActivity("sp1");	
	proj = psh.create();
	proj.initProject("workCancel");
	eng.terminateActivity("init");
	eng.cancelActivity("sp1");
	
	assertTrue("BnNode State Error init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	assertTrue("BnNode State Error sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.DEAD);
	assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.DEAD);
	assertTrue("BnNode State Error ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.DEAD);
	assertTrue("BnEdge State Error e3",proj.getEdgeValue(e3).getState() == Constants.Ed.DEAD);
	assertTrue("BnNode State Error ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.DEAD);
	
	eng.terminate();
	assertTrue("Wrong project state",	proj.getDetails().getState() == Constants.Pj.TERMINATED);						
    }
    
    public void testCancelORActivity() throws Exception {
	ProjectSession proj;
	BnProjectValue pm;
	BnNodeValue nd;
	BnEdgeValue ev;
	
	// creation du projet
	proj = psh.create();
	proj.initProject("workCancelOR");
	proj.addNode("init", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("sp1", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("ac1", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("ac2", Constants.Nd.AND_JOIN_NODE);
    	proj.addNode("ac3", Constants.Nd.OR_JOIN_NODE);
	
	String e1 = proj.addEdge("init", "sp1");
	String e2 = proj.addEdge("sp1", "ac1");
	String e3 = proj.addEdge("sp1", "ac2");
	String e4 = proj.addEdge("ac1", "ac3");
	String e5 = proj.addEdge("ac2", "ac3");
	
	proj.addRole("test", "testeur");
	proj.setUserRole("admin", "test");
	
	eng = eh.create();
	eng.initProject("workCancelOR");
	eng.startActivity("init");
	eng.startActivity("sp1");
	eng.terminateActivity("init");
	eng.terminateActivity("sp1");
	eng.startActivity("ac2");
	eng.cancelActivity("ac1");
	
	assertTrue("BnNode State Error init",proj.getNodeValue("init").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	assertTrue("BnNode State Error sp1",proj.getNodeValue("sp1").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
	assertTrue("BnNode State Error ac1",proj.getNodeValue("ac1").getState() == Constants.Nd.DEAD);
	assertTrue("BnEdge State Error e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
	assertTrue("BnNode State Error ac2",proj.getNodeValue("ac2").getState() == Constants.Nd.EXECUTING);
	assertTrue("BnEdge State Error e4",proj.getEdgeValue(e4).getState() == Constants.Ed.DEAD);
	assertTrue("BnEdge State Error e5",proj.getEdgeValue(e5).getState() == Constants.Ed.ANTICIPATING);
	assertTrue("BnNode State Error ac3",proj.getNodeValue("ac3").getState() == Constants.Nd.ANTICIPABLE);
    }
    
    public void testAutomaticNode() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("testoto");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	proj.addEdge("nonode", "supernonode");
	Engine eng = eh.create();
	eng.initProject("testoto");
	eng.startActivity("nonode");
	assertTrue("Supernonode will be terminated", proj.getNodeValue("supernonode").getState()== Constants.Nd.ANTICIPATING);
	eng.terminateActivity("nonode");
	assertTrue("Supernonode will be terminated", proj.getNodeValue("supernonode").getState()== Constants.Nd.TERMINATED);
    }
    
    public void testAutomaticNodeOR() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("testoto2");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("nonode2", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.OR_JOIN_AUTOMATIC_NODE);
	proj.addEdge("nonode", "supernonode");
	proj.addEdge("nonode2", "supernonode");
	Engine eng = eh.create();
	eng.initProject("testoto2");
	eng.startActivity("nonode");
	assertTrue("Supernonode will be anticipating ", proj.getNodeValue("supernonode").getState()== Constants.Nd.ANTICIPATING);
	eng.startActivity("nonode2");
	eng.terminateActivity("nonode");
	assertTrue("Supernonode will be anticipating", proj.getNodeValue("supernonode").getState() == Constants.Nd.TERMINATED);
    }
    
    public void testTraditionalActivityNode() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("testtradacti");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.AND_JOIN_NODE);
	proj.setNodeTraditional("supernonode");
	proj.addEdge("nonode", "supernonode");
	Engine eng = eh.create();
	eng.initProject("testtradacti");
	eng.startActivity("nonode");
	assertTrue("Supernonode will be initial", proj.getNodeValue("supernonode").getState()== Constants.Nd.INITIAL);
	eng.terminateActivity("nonode");
	assertTrue("Supernonode will be ready",proj.getNodeValue("supernonode").getState() == Constants.Nd.READY);
    }
    
    public void testTraditionalAutomaticNode() throws Exception {	
	ProjectSession proj = psh.create();
	proj.initProject("testtrad");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	proj.setNodeTraditional("supernonode");
	proj.addEdge("nonode", "supernonode");
	Engine eng = eh.create();
	eng.initProject("testtrad");
	eng.startActivity("nonode");
	assertTrue("Supernonode will be initial",proj.getNodeValue("supernonode").getState()== Constants.Nd.INITIAL);
	eng.terminateActivity("nonode");
	assertTrue("Supernonode will be terminated",proj.getNodeValue("supernonode").getState()== Constants.Nd.TERMINATED);
    }
    
    public void testEdgeConditionPropagation() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("edgeCondP");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.AND_JOIN_NODE);
	String e1 = proj.addEdge("nonode", "supernonode");
	proj.setEdgeCondition(e1,"((edge.getInBnNode()).getName()).equals(\"nonode\")");
	Engine eng = eh.create();
	eng.initProject("edgeCondP");
	eng.startActivity("nonode");
	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.EXECUTING);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.ANTICIPABLE);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ANTICIPATING);
	eng.terminateActivity("nonode");
	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.READY);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
    }

    public void testEdgeConditionPropagationProperty() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("edgeCondPP");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.AND_JOIN_NODE);
	proj.setNodeProperty("nonode","prop1","value1",true);
	String e1 = proj.addEdge("nonode", "supernonode");
	proj.setEdgeCondition(e1,"prop1.equals(\"value1\")");
	Engine eng = eh.create();
	eng.initProject("edgeCondPP");
	eng.startActivity("nonode");
	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.EXECUTING);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.ANTICIPABLE);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ANTICIPATING);
	eng.terminateActivity("nonode");
	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.READY);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
    }

    public void testEdgeConditionNotPropagation() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("edgeCondNP");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.AND_JOIN_NODE);
	String e1 = proj.addEdge("nonode", "supernonode");
	proj.setEdgeCondition(e1,"((edge.getInBnNode()).getName()).equals(\"supernonode\")");
	Engine eng = eh.create();
	eng.initProject("edgeCondNP");
	eng.startActivity("nonode");
	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.EXECUTING);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.ANTICIPABLE);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ANTICIPATING);
	eng.terminateActivity("nonode");
	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.DEAD);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.DEAD);
    }

    public void testEdgeConditionANDSplitPropagation() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("edgeCondSP");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("nonode1", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("nonode2", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.AND_JOIN_NODE);
	String e1 = proj.addEdge("nonode", "nonode1");
	String e2 = proj.addEdge("nonode1", "nonode2");
	String e3 = proj.addEdge("nonode1", "supernonode");
	proj.setEdgeCondition(e1,"((edge.getInBnNode()).getName()).equals(\"nonode\")");
	proj.setEdgeCondition(e2,"((edge.getInBnNode()).getName()).equals(\"nonode1\")");
	proj.setEdgeCondition(e3,"((edge.getInBnNode()).getName()).equals(\"nonode1\")");
	Engine eng = eh.create();
	eng.initProject("edgeCondSP");
	eng.startActivity("nonode");
	eng.startActivity("nonode1");

	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.EXECUTING);
	assertTrue("BnNode State Error nonode1",proj.getNodeValue("nonode1").getState() == Constants.Nd.ANTICIPATING);
	assertTrue("BnNode State Error nonode2",proj.getNodeValue("nonode2").getState() == Constants.Nd.ANTICIPABLE);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.ANTICIPABLE);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ANTICIPATING);
	assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ANTICIPATING);
	assertTrue("BnEdge State Error e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ANTICIPATING);

	eng.terminateActivity("nonode");
	eng.terminateActivity("nonode1");

	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnNode State Error nonode1",proj.getNodeValue("nonode1").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnNode State Error nonode2",proj.getNodeValue("nonode2").getState() == Constants.Nd.READY);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.READY);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ACTIVE);
	assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
	assertTrue("BnEdge State Error e3",proj.getEdgeValue(e3).getState() == Constants.Ed.ACTIVE);
    }

    public void testEdgeConditionORSplitPropagation() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("edgeCondOP");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("nonode1", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("nonode2", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("supernonode", Constants.Nd.AND_JOIN_NODE);
	String e1 = proj.addEdge("nonode", "nonode1");
	String e2 = proj.addEdge("nonode", "nonode2");
	String e3 = proj.addEdge("nonode", "supernonode");
	proj.setEdgeCondition(e1,"((edge.getInBnNode()).getName()).equals(\"nonode1\")");
	proj.setEdgeCondition(e2,"((edge.getInBnNode()).getName()).equals(\"nonode\")");
	proj.setEdgeCondition(e3,"((edge.getInBnNode()).getName()).equals(\"nonode1\")");
	Engine eng = eh.create();
	eng.initProject("edgeCondOP");
	eng.startActivity("nonode");
	eng.terminateActivity("nonode");

	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.TERMINATED);
	assertTrue("BnNode State Error nonode1",proj.getNodeValue("nonode1").getState() == Constants.Nd.DEAD);
	assertTrue("BnNode State Error nonode2",proj.getNodeValue("nonode2").getState() == Constants.Nd.READY);
	assertTrue("BnNode State Error supernonode",proj.getNodeValue("supernonode").getState() == Constants.Nd.DEAD);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.DEAD);
	assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.ACTIVE);
	assertTrue("BnEdge State Error e3",proj.getEdgeValue(e3).getState() == Constants.Ed.DEAD);
    }

    public void testExternalAgentWithANDJoinNode() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("externalAgent");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("nonode1", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("nonode2", Constants.Nd.AND_JOIN_NODE);
	proj.addAgent("superagent");
	String e1 = proj.addEdge("nonode", "nonode1");
	String e2 = proj.addEdge("nonode1", "nonode2");
	String e3 = proj.addAgentEdge("superagent", "nonode1");
	Engine eng = eh.create();
	eng.initProject("externalAgent");
	eng.startActivity("nonode");
	eng.activeAgent("superagent");

	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.EXECUTING);
	assertTrue("BnNode State Error nonode1",proj.getNodeValue("nonode1").getState() == Constants.Nd.ANTICIPABLE);
	assertTrue("BnNode State Error nonode2",proj.getNodeValue("nonode2").getState() == Constants.Nd.INITIAL);
	assertTrue("BnNode State Error superagent",proj.getAgentValue("superagent").getState() == Constants.Ag.ACTIVE);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.ANTICIPATING);
	assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.INITIAL);
	assertTrue("BnEdge State Error e3",proj.getAgentEdgeValue(e3).getState() == Constants.Ed.ANTICIPATING);
    }

    public void testExternalAgentWithORJoinNode() throws Exception {
	ProjectSession proj = psh.create();
	proj.initProject("externalAgentOr");
	proj.addNode("nonode", Constants.Nd.AND_JOIN_NODE);
	proj.addNode("nonode1", Constants.Nd.OR_JOIN_NODE);
	proj.addNode("nonode2", Constants.Nd.AND_JOIN_NODE);
	proj.addAgent("superagent");
	String e1 = proj.addEdge("nonode", "nonode1");
	String e2 = proj.addEdge("nonode1", "nonode2");
	String e3 = proj.addAgentEdge("superagent", "nonode1");
	Engine eng = eh.create();
	eng.initProject("externalAgentOr");
	eng.activeAgent("superagent");

	assertTrue("BnNode State Error nonode",proj.getNodeValue("nonode").getState() == Constants.Nd.READY);
	assertTrue("BnNode State Error nonode1",proj.getNodeValue("nonode1").getState() == Constants.Nd.ANTICIPABLE);
	assertTrue("BnNode State Error nonode2",proj.getNodeValue("nonode2").getState() == Constants.Nd.INITIAL);
	assertTrue("BnNode State Error superagent",proj.getAgentValue("superagent").getState() == Constants.Ag.ACTIVE);
	assertTrue("BnEdge State Error e1",proj.getEdgeValue(e1).getState() == Constants.Ed.INITIAL);
	assertTrue("BnEdge State Error e2",proj.getEdgeValue(e2).getState() == Constants.Ed.INITIAL);
	assertTrue("BnEdge State Error e3",proj.getAgentEdgeValue(e3).getState() == Constants.Ed.ANTICIPATING);
    }
	
}

