package hero.client.test;

/*
*
* NodeTests.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.hook.Hook;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHome;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnUser;
import hero.interfaces.BnUserHome;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;
import hero.util.DuplicatedEdgeException;
import hero.util.HeroException;

import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;
import junit.framework.TestSuite;

public class ProjectSessionTests extends TestCase {
    private BnProjectHome projecth;
    private ProjectSessionHome projectSessionh;

    public ProjectSessionTests(String testname) {
	super(testname);
    }
    
    public static TestSuite suite() {
	return new TestSuite(ProjectSessionTests.class);
    }

    public void setUp() throws Exception {
    }

    public void testCreateProjectSession() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectsession");
	BnProjectValue pv=lProject.getDetails();
	BnUserLightValue[] uv=pv.getBnUsers();
	int i=0;
	while (i<uv.length) {
	    if (uv[i].getName().equals("admin"))
		break;
	    i++;
	}
	assertTrue("Admin fait partie du projet",i<uv.length);
    }

    public void testCloneProjectSession() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectsource");
	lProject.addNode("nclone",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c1",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c2",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone","admin");
	lProject.setNodeRole("c1","admin");
	lProject.setNodeRole("c2","admin");
	String edge1=lProject.addEdge("c1","c2");
	lProject = lHome.create();
	lProject.initProject("projectsource","tralara");	
	BnProjectValue pv=lProject.getDetails();
	assertTrue("Clone failure1",pv.getBnNodes().length==3);	  
	assertTrue("Clone failure2",pv.getName().equalsIgnoreCase("tralara"));	            
    }
   public void testImportProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectS");
	lProject.addNode("nclone",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c1",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c2",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone","admin");
	lProject.setNodeRole("c1","admin");
	lProject.setNodeRole("c2","admin");
	String edge1=lProject.addEdge("c1","c2");

	lProject = lHome.create();
	lProject.initProject("pDest");
	
	lProject.addNode("nclone",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone","admin");
	lProject.importProject("projectS","pDest");
	BnProjectValue pv=lProject.getDetails();
	assertTrue("Clone failure1",pv.getBnNodes().length==4);	  
	assertTrue("Clone failure2",pv.getName().equalsIgnoreCase("pDest"));	            
    }

    public void testAddUserToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectadduser");	
	BnUserHome uHome = BnUserUtil.getHome();
	BnUserValue pdata=new BnUserValue();
	pdata.setName("bidochon");
	BnUser lUser = uHome.create(pdata);
	lProject.addUser("bidochon");
	pdata.setName("marcel");
	lUser = uHome.create(pdata);
	lProject.addUser("marcel");
	pdata.setName("germaine");
	lUser = uHome.create(pdata);
	lProject.addUser("germaine");
	assertTrue(lProject.containsUser("bidochon"));
    }

    public void testAddNodeToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectaddnode");	
	lProject.addNode("thenode",1);
	BnNodeValue nv=lProject.getNodeValue("thenode");
	assertTrue("Creator not correctly set",nv.getCreator().getName().equals("admin"));
    }

    public void testAddEdgeToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectaddedge");		
	lProject.addNode("thenode",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("thenode2",Constants.Nd.AND_JOIN_NODE);
	String ename=lProject.addEdge("thenode","thenode2");
	BnEdgeValue ev=lProject.getEdgeValue(ename);
	BnNodeValue nv=lProject.getNodeValue("thenode2");
	assertTrue("addEdge failed a1",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("bad node state test3",nv.getState()==Constants.Nd.INITIAL);    }

    public void testProjectSessionAddUser() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("pst");		

	try {
	    pss.addUser("badUser");
	    assertTrue("Should have throw EJBException",false);
	} catch (Exception e) {
	    //expected behavior
	}
    }	

    public void testProjectSessionAddNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("pst2");	
	pss.addNode("test",Constants.Nd.AND_JOIN_NODE);
	BnNodeValue pv=pss.getNodeValue("test");

	assertTrue("Creator not correctly set",pv.getCreator().getName().equals("admin"));
    }

    public void testProjectSessionAddEdgesState() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("w");		

	pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	String edname=pss.addEdge("node1","node2");
	BnEdgeValue ev=pss.getEdgeValue(edname);
	BnNodeValue nv=pss.getNodeValue("node2");
	assertTrue("addEdge failed a1",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("addEdge failed a1/n2",nv.getState()==Constants.Nd.INITIAL);
	edname=pss.addEdge("node2","node3");
	ev=pss.getEdgeValue(edname);
	BnNodeValue nv2=pss.getNodeValue("node2");
	BnNodeValue nv3=pss.getNodeValue("node3");
	assertTrue("addEdge failed a2",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("addEdge failed a2/nv2",nv2.getState()==Constants.Nd.INITIAL);
	assertTrue("addEdge failed a2/nv3",nv3.getState()==Constants.Nd.INITIAL);
    }

    public void testcheckDuplicateEdge() throws Exception {
    
	try {
	    ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession pss = projectSessionh.create();
		pss.initProject("wde");		    
	    
	    pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	    pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node1","node2");
	    
	    pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node2","node3");
	    
	    pss.addNode("node4",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node1","node4");
	    
	    pss.addEdge("node1","node2");
	    assertTrue("Should have thrown a DuplicatedEdgeException",false);
	}catch(DuplicatedEdgeException e){}
    }

    public void testDynamicProject() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("dynamic");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n1","admin");

	UserSession usr=usersh.create();

	usr.startActivity("dynamic","n1");

	pss.addNode("n3",Constants.Nd.AND_JOIN_NODE);
	String edge2=pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPABLE);
	assertTrue("dynamic change failed - n3",pss.getNodeValue("n3").getState()==Constants.Nd.INITIAL);

	usr.startActivity("dynamic","n2");

	assertTrue("dynamic change failed - n1.1",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.1",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.1",pss.getNodeValue("n3").getState()==Constants.Nd.ANTICIPABLE);

	pss.deleteEdge(edge2);

	assertTrue("dynamic change failed - n1.2",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.2",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.2",pss.getNodeValue("n3").getState()==Constants.Nd.READY);

	pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1.3",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.3",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.3",pss.getNodeValue("n3").getState()==Constants.Nd.ANTICIPABLE);

    }
	
    public void TestDynamicProject2() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();

	ProjectSession pss = projectSessionh.create();
	pss.initProject("dynamic2");	

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");
	
	UserSession usr=usersh.create();

	usr.startActivity("dynamic2","n1");

	pss.addNode("n3",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1",pss.getEdgeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2",pss.getEdgeValue("n2").getState()==Constants.Nd.ANTICIPABLE);
	assertTrue("dynamic change failed - n3",pss.getEdgeValue("n3").getState()==Constants.Nd.INITIAL);

	pss.addNode("n4",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n1","n4");
	pss.deleteEdge(edge1);

	assertTrue("dynamic change failed - n1.1",pss.getEdgeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.1",pss.getEdgeValue("n2").getState()==Constants.Nd.READY);
	assertTrue("dynamic change failed - n3.1",pss.getEdgeValue("n3").getState()==Constants.Nd.INITIAL);
	assertTrue("dynamic change failed - n4.1",pss.getEdgeValue("n4").getState()==Constants.Nd.ANTICIPABLE);
    }

    public void testDeleteNode() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	
	ProjectSession pss = projectSessionh.create();
	pss.initProject("deleteNode");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");

	try {
	    pss.deleteNode("n2");
	} catch (HeroException he) { 
	    assertTrue("Deletion Failed",false); 
	}
	
	UserSession usr=usersh.create();

	usr.startActivity("deleteNode","n1");
	try {
	    pss.deleteNode("n1");
	    assertTrue("Deletion Should have Failed",false); 
	} catch (HeroException he) { // Normal Behavior
	}
    }	


    public void testPropertyNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");
	pss.setNodeProperty("n1","kpn1","vpn1");
	pss.setNodeProperty("n2","kpn2","vpn2");
	pss.setNodeProperty("n2","kpn2","vpn22");
	
	pss.deleteNode("n2");
	
    }	

    public void testDeletePropertyNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.deleteNodeProperty("n1","kpn1");
	
    }	

    public void testPropertyProject() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.setProperty("maintenant","ara");
	pss.setProperty("deja","ahora");
	pss.setProperty("deja","ja");
	
    }	
    public void testDeletePropertyProject() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.deleteProperty("maintenant");	
    }	


    public void testNodePropertyPropagate() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.addNode("n5",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n6",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n5","n6");
	pss.setNodeRole("n5","admin");
	pss.setNodeRole("n6","admin");
	pss.setNodeProperty("n5","value","15");
	pss.setNodeProperty("n5","value2","10");
	pss.setNodeProperty("n6","value","25");

	UserSession usr=usersh.create();
	usr.startActivity("propertyNode","n5");
	usr.terminateActivity("propertyNode","n5");
    }

  public void testGetPropertiesNode() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.addNode("p1",Constants.Nd.AND_JOIN_NODE);

	pss.setNodeProperty("p1","k1","10");
	
	Collection c=pss.getNodeProperties("p1");
        
	for(Iterator i=c.iterator();i.hasNext();){
	    BnNodePropertyValue p=(BnNodePropertyValue)i.next();
	    String key=p.getTheKey();
	    String value=p.getTheValue();
	    assertTrue("Get BnNode Properties",key.equalsIgnoreCase("k1"));
	    
	}
    }
  public void testGetProperties() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.setProperty("pk1","xxx");
	
	Collection c=pss.getProperties();
        
	for(Iterator i=c.iterator();i.hasNext();){
	    BnProjectPropertyValue p=(BnProjectPropertyValue)i.next();
	    String key=p.getTheKey();
	    String value=p.getTheValue();
	    assertTrue("Get Properties",key.equalsIgnoreCase("pk1"));
	    
	}
    }

 public void testNodeDeadline() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("deadlineNode");	
		
	pss.addNode("d1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("d1","admin");
	pss.setNodeDeadline("d1",System.currentTimeMillis()+(long)10000);	
    }

   public void testNodeInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterNode");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnNodee Hook test, node: \"+n.getName());"
	    + "}";
	pss.addNodeInterHook("h1","interTest",hero.interfaces.Constants.Nd.AFTERSTART,Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterNode","h1");
	usr.terminateActivity("hookInterNode","h1");	
   }

   public void testNodeInterHookProperty() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterNodeP");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeProperty("h1","testp1","testv1",true);
	pss.setNodeProperty("h1","testp2","testv2",true);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnNodee Hook properties tests, node: \"+n.getName());\n"
	    + "System.out.println(\"property \"+testp2);\n"
	    + "}";
	pss.addNodeInterHook("h1","interTest",hero.interfaces.Constants.Nd.AFTERSTART,Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterNodeP","h1");
	usr.terminateActivity("hookInterNodeP","h1");
   }
    
    public void testProjectInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterProject");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnProjectt Hook test, node: \"+n.getName());"
	    + "}";
	pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterProject","h1");
	usr.terminateActivity("hookInterProject","h1");	
   }

    public void testDuplicatedInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("dupliHookInter");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnProjectt Hook test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
	    + "}";
	pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Hook.BSINTERACTIVE,script);
	try {
	    pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Hook.BSINTERACTIVE,script);
	    assertTrue("should have failed", false);
	} catch (HeroException h) {
	    // normal behavior
	}
	usr.startActivity("dupliHookInter","h1");
	usr.terminateActivity("dupliHookInter","h1");	
   }

    

}
