package hero.jabber;

/*
*
* JabberServiceMBean.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import javax.management.*;
// Jabber imports
import com.echomine.jabber.*;
import com.echomine.net.*;
import java.util.*;
import javax.naming.Context;
import javax.naming.NamingException;
import org.objectweb.jonas.service.ServiceException;

//import objectweb
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceManager;


public class JabberService extends AbsServiceImpl implements JabberServiceMBean {


	JabberSession jabberSession;
	JabberContext jabberContext;
	// Constants -----------------------------------------------------
 
        
 
	// Attributes ----------------------------------------------------

	private String j_user;    
	private String j_password;
	private String j_server;
    private int j_port;

	/**
	 * Reference to a MBean server.
	 */
	private MBeanServer mbeanServer = null;

	public final static String SERVICE_NAME = "jonas:type=service,name=Jabber";

	// Public --------------------------------------------------------
 
  
 
	public void doInit(Context ctx) throws ServiceException {
		// Get the JMX Server via JMX Service
		try {
			mbeanServer = 
				((JmxService) ServiceManager.getJmxService()).getJmxServer();
		} catch (ServiceException e) {
			// the JMX service may not be started
			mbeanServer = null;
		}
	try {
			this.j_server = (String) ctx.lookup("jonas.service.jabberService.host");
			this.j_port = new Integer((String) ctx.lookup("jonas.service.jabberService.port")).intValue();
			this.j_user = (String) ctx.lookup("jonas.service.jabberService.user");
			this.j_password = (String) ctx.lookup("jonas.service.jabberService.password");
		} catch (NamingException e) {}
	}

	public void setUser(String user) {j_user = user;}  
	protected String getUser() {return j_user;}     
	public void setPassword(String password) {j_password = password;}  
	protected String getPassword() {return j_password;}
	public void setServer(String server) {j_server = server;}  
	protected String getServer() {return j_server;}
	public void setPort(int port) {j_port = port;}  
	protected int getPort() {return j_port;}

	public void doStart() throws ServiceException {

		try {
			// Register MailService MBean : MailServiceImplMBean
			if (mbeanServer != null) {
				mbeanServer.registerMBean(this,new ObjectName(SERVICE_NAME));
			}
		} catch (InstanceAlreadyExistsException iae) {
			throw new ServiceException("Cannot start the Jabber Service (Already Exists)" , iae);
		} catch (MBeanRegistrationException mbre) {
			throw new ServiceException("Cannot start the Jabber Service (MBean registration error)" , mbre);
		} catch (NotCompliantMBeanException ncmbe) {
			throw new ServiceException("Cannot start the Jabber Service (MBean Not compliant error)" , ncmbe);
		} catch (MalformedObjectNameException mone) {
			throw new ServiceException("Cannot start the Jabber Service (ObjectName Malformed)" , mone);
		}


	System.out.println("JabberService :: Start Service");
	this.setName("Jabber Service");
	// Initialize the Jabber context
	jabberContext = new JabberContext(j_user,j_password,j_server);

	// Create an instance of the Jabber session factory
	Jabber jabber = new Jabber();
	// Create the new session
	jabberSession = jabber.createSession(jabberContext);
	// Connect to the server
	try{
		jabberSession.connect(j_server,j_port);
	}catch(ConnectionFailedException cf){
		System.err.println(" Jabber service -> Connection Failed:"+cf.getMessage());
	}catch(Exception e){
		System.err.println(" Jabber service -> Connection Failed:"+e.getMessage()); 
	}
/*	try{
            jabberSession.getUserService().login();
  	}catch(JabberMessageException jm){
		System.err.println(" Jabber service -> Login Failed:"+jm.getMessage());
	}catch(Exception e){
		System.err.println(" Jabber service -> Login Failed:"+e.getMessage());
	}*/

	}

	/**
	* Send a Jabber message to the specified JID
	*
	*/
	public synchronized boolean sendMessage(String JID, String resource, String subject, String messageBody){
	try{
		JabberChatMessage msg = new JabberChatMessage(JabberChatMessage.TYPE_NORMAL);
		msg.setSubject(subject);
		msg.setBody(messageBody);
		msg.setTo(JID);
		jabberSession.sendMessage(msg);
		return true;
	}catch(Exception e){System.err.println("Jabber service -> Send Message Failed:"+e.getMessage());}
	return false;
	}


	/**
	 * Register a new user to Jabber server
	 *
	 */
	public synchronized boolean registerUser(String login, String password, String email, String name){
	boolean status=true;
	jabberSession.disconnect();
	Jabber jabber = new Jabber();
	JabberSession session = jabber.createSession(jabberContext);
	try {
		session.connect(j_server, j_port); 
			HashMap fields = new HashMap();
			fields.put("username", login);
			fields.put("password", password);
			fields.put("email", email);
			fields.put("name", name);
		try{
		session.getUserService().register(jabberContext.getServerName(), fields);
		}catch(JabberMessageException ex){ 
		status=false;
		System.err.println("Jabber service ->"+ex);
		}
		session.disconnect();
		start();
		return status;
		}catch(Exception e){System.err.println("Jabber service ->"+e);}
		start();
	return false;

	}

	/**
	 * Add to Roster
	 *
	 */
	public synchronized boolean addToRoster(String login, String password, String contactJID,String contactName, String contactGroup ){
	boolean status=true;
	jabberSession.disconnect();
	Jabber jabber = new Jabber();
	JabberContext context = new JabberContext(login,password,j_server);
	JabberSession session = jabber.createSession(context);
	try {
		session.connect(j_server, j_port); 
		session.getUserService().login();
		try{
		session.getRosterService().addToRoster(contactJID,contactName,contactGroup,true);
		session.getPresenceService().subscribe(contactJID);
		}catch(Exception ex){ 
		status=false;
		System.err.println("Jabber service ->"+ex);
		}
		session.disconnect();
		start();
		return status;
		}catch(Exception e){System.err.println("Jabber service ->"+e);}
	start();
	return false;

	}
    
	public void doStop() throws ServiceException {
	  if (mbeanServer != null) {
			try {
				mbeanServer.unregisterMBean(new ObjectName(SERVICE_NAME));
			} catch (Exception e) {
				throw new ServiceException("Cannot stop the Jabber Service (JMX).", e);
			}
		}
		System.out.println("Jabber :: Stop Service");
		jabberSession.disconnect();
	}
}
