/*
* 02/01/2002 - 15:24:07
*
* NotificationBean.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package hero.message;

// MDB imports
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.util.EventConstants;
import hero.util.HeroException;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.mail.Address;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

/**
 * This is a Message Driven Bean
 *
 * @ejb:message-driven
 * @ejb:bean name="NotificationBean"
 *           transaction-type="Container"
 *           acknowledge-mode="Auto-acknowledge"
 *           destination-type="javax.jms.Topic"
 *           subscription-durability="NonDurable"
 *
 * @jboss:destination-jndi-name name="testTopic"
 * @ejb.security-identity run-as="Admin"
 * @ejb.permission unchecked="yes"   
 * 
 * 
 * @jonas.message-driven-destination jndi-name="testTopic"
 * @jonas.bean  ejb-name="NotificationBean"    
 *              jndi-name="testTopic"
 * @ejb.transaction type= "Required"
 *
 * @author $Author: Oriol Montala $
 * @version $Revision: 0.1 $
 */

public class NotificationBean implements MessageDrivenBean, MessageListener{  

    //private MessageDrivenContext ctx = null;
    private transient MessageDrivenContext ctx;
    private Session session;

    
    private void initMailService()
    {
	try {
	    session = (Session)PortableRemoteObject.narrow(new InitialContext().lookup("Mail"), Session.class);
	} catch (javax.naming.NamingException e) {
	    e.printStackTrace();
	}
    }
    
    private void sendMail(String toAccount, String title,String body) throws Exception {
	try {
	    MimeMessage m = new MimeMessage(session);
	    m.setFrom();
	    Address[] to = new InternetAddress[] {new InternetAddress(toAccount) };
	    m.setRecipients(javax.mail.Message.RecipientType.TO, to);
	    m.setSubject(title);
	    m.setSentDate(new java.util.Date());
	    m.setContent(body,"text/plain");
	    Transport.send(m);
	} catch (javax.mail.MessagingException e) {
	    System.out.println("mail-service.xml configuration error: "+e);
	}                
     }
    
    private void sendJabber(String toAccount, String subject, String message) throws Exception{
	//ObjectName jabber    = new ObjectName("jboss:service=Jabber"); 
	ObjectName jabber    = new ObjectName("jonas:type=service,name=Jabber");
	List list = MBeanServerFactory.findMBeanServer(null);
	MBeanServer mServer = (MBeanServer)list.iterator().next();    
	Object id = (Object) mServer.invoke( jabber,
						 "sendMessage",
						 new Object[] { 
						     toAccount, //JID 
						     null  ,    //Resource
						     subject,   //Subject
						     message    //message
						 },
						 new String[] {
						     "".getClass().getName(),
						     "".getClass().getName(), 
						     "".getClass().getName(), 
						     "".getClass().getName()
						 }
					      );
    }


    public void setMessageDrivenContext(MessageDrivenContext ctx)
	throws EJBException
    {
    	this.ctx = ctx;
	initMailService();
    }

    private Collection usersRegistered(String project, String event) throws Exception {
	Collection c= new ArrayList();
	ProjectSessionLocalHome home=ProjectSessionUtil.getLocalHome();
	ProjectSessionLocal prj = home.create();
	prj.initProject(project);
	BnProjectValue pv=prj.getDetails();
	BnUserLightValue[] users=pv.getBnUsers();
	for(int i=0;i<users.length;i++){
	    if(true){
		c.add(users[i]);
	    } 
	}
	return c;
    } 

    public void ejbCreate()
    { 
    }

    public void ejbRemove() 
    { 
	//    	ctx=null;
    }
    
    public void onMessage(Message message)    
    { 
	BnUserLocal mUser;
	try{
	    String event= (String)message.getObjectProperty(EventConstants.EVENT); 
	    String projectName = (String)message.getObjectProperty(EventConstants.PROJECTNAME);   
	    if(event.equalsIgnoreCase(EventConstants.START) || event.equalsIgnoreCase(EventConstants.TERMINATED)){ 
		Collection c=usersRegistered(projectName,event);
		for(Iterator i=c.iterator();i.hasNext();){
		    BnUserLightValue userLight=(BnUserLightValue)i.next();
		    try {
			BnUserLocalHome userhome=hero.interfaces.BnUserUtil.getLocalHome();
			mUser=userhome.findByName(userLight.getName());
		    } catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		    } catch (javax.ejb.FinderException fe) {
			throw new HeroException(userLight.getName()+" does not exists");
		    }
		    Collection props=mUser.getBnProperties();
		    for(Iterator it=props.iterator();it.hasNext();){
			BnUserPropertyLocal pValue= (BnUserPropertyLocal)it.next();
			if((pValue.getTheKey()).equalsIgnoreCase(event+"Mail") && pValue.getTheValue()!= null){
			    if(userLight.getEmail()!=null)
				sendMail(userLight.getEmail(),"Event Notification","BnProject:"+projectName+" Event:"+event);
			}
			if((pValue.getTheKey()).equalsIgnoreCase(event+"Jabber") && pValue.getTheValue()!= null ){
			    String jab=userLight.getJabber();
			    if(jab!=null){				    
				sendJabber(jab,"Bon dia","Hola:"+projectName+" Event:"+event);
			    }
			}    
		    }
		}
	    }  
	}catch(Exception e1){e1.printStackTrace();}  	
    }
    
}
