package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* UserRegistrationEJB.java - 
* Copyright (C) 2002 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnAuthRoleLocal;
import hero.interfaces.BnAuthRoleLocalHome;
import hero.interfaces.BnAuthRoleUtil;
import hero.interfaces.BnAuthRoleValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.BnUserPropertyLocalHome;
import hero.interfaces.BnUserPropertyUtil;
import hero.interfaces.BnUserUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.InvalidValueException;
import hero.util.HeroException;

import java.rmi.RemoteException;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Category;

/**
 * Session Bean Template
 *
 * @ejb:bean name="UserRegistration"
 *           display-name="UserRegistration Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/UserRegistration"
 *           local-jndi-name="ejb/hero/UserRegistration_L"
 *
 * @ejb:ejb-ref ejb-name="UserRegistration"
 *              ref-name="myhero/UserRegistration"
 * @ejb:transaction type="Required"
 * @ejb:transaction-type type="Container"
 * @ejb.permission unchecked="yes"  
 * @ejb.security-identity run-as="Admin"
 * @jonas.bean
 *      ejb-name="UserRegistration"
 *      jndi-name="ejb/hero/UserRegistration"
 *
 **/

public class UserRegistrationBean implements SessionBean {
  
 // Utility variable
    private static final Category log = Category.getInstance(UserRegistrationBean.class);

   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;

    private BnUserLocalHome userhome;
    private BnAuthRoleLocalHome rolehome;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------  

   /**
   * Create the BnUser Session Bean
   *
   * @throws CreateException 
   *
   * @ejb:create-method view-type="both"
   **/
    public void ejbCreate() throws CreateException {
	log.debug("create");
    }

    /**
     * @param name name of the new user
     * @param password password of the new user
     * @param email email of the new user
     * @ejb:interface-method view-type="both"
     *
     **/
    public void userCreate(String name, String password, String email) throws HeroException {
	try {
	    BnUserValue newUser=new BnUserValue();
	    newUser.setName(name);
	    newUser.setPassword(password);
	    newUser.setEmail(email);
	    BnUserLocal user = userhome.create(newUser);
	    this.setUserRole(name,"Admin");
	} catch (InvalidValueException ie) {
	    throw new HeroException(ie.getMessage());
	} catch (CreateException f) {
	    throw new HeroException(name+" is already exist");
	}
    }

    /**
     * @param name name of the new user
     * @param password password of the new user
     * @param email email of the new user
     * @param jabber jabber address of the new user
     * @ejb:interface-method view-type="both"
     *
     **/

    public void userCreate(String name, String password, String email, String jabber) throws HeroException {
	try {
	    BnUserValue newUser=new BnUserValue();
	    newUser.setName(name);
	    newUser.setPassword(password);
	    newUser.setEmail(email);
	    newUser.setJabber(jabber);
	    BnUserLocal user = userhome.create(newUser);
	    this.setUserRole(name,"Admin");
	} catch (InvalidValueException ie) {
	    throw new HeroException(ie.getMessage());
	} catch (CreateException f) {
	    throw new HeroException(name+" is already exist");
	}
    }

    /**
     * @param name name of the new role
     * @param roleGroup
     * @param email email of the new user
     * @ejb:interface-method view-type="both"
     *
     **/
    public void roleCreate(String name, String roleGroup) throws HeroException {
	try {
	    BnAuthRoleValue newRole=new BnAuthRoleValue();
	    newRole.setName(name);
	    newRole.setBnRoleGroup(roleGroup);
	    BnAuthRoleLocal role = rolehome.create(newRole);
	} catch (CreateException f) {
	    throw new HeroException(name+" is already exist");
	}
    }

    /**
     * @param username name of the user
     * @param rolename name of the role
     * @ejb:interface-method view-type="both"
     *
     **/
    public void setUserRole(String userName, String roleName) throws HeroException {
	log.debug("setUserRole");
	try {
	    BnUserLocal user = userhome.findByName(userName);
	    BnAuthRoleLocal role = rolehome.findByName(roleName);
	    Collection roles = user.getBnAuthRoles();
	    roles.add(role);
	} catch (Exception f) {
	    throw new HeroException(userName+" cannot exist");
	}
	log.debug("setUserRole - end");
    }

    /**
     * @param key of the user property
     * @param value of the user property
     * @ejb:interface-method view-type="both"
     *
     **/
    public void setUserProperty(String userName, String key, String value) throws HeroException {
	log.debug("setUserProperty");

	BnUserLocalHome userhome;
	BnUserLocal mUser;
	BnUserPropertyLocalHome pHome;
	BnUserPropertyLocal propertyLocal=null;

	try {
	    userhome=BnUserUtil.getLocalHome();
	    mUser=userhome.findByName(userName);
	
	    try {
		pHome=BnUserPropertyUtil.getLocalHome();
	    } catch (javax.naming.NamingException ne) {
		throw new HeroException(ne.getMessage());
	    }
	    Collection c=null;
	    try {
		c=pHome.findByTheKey(mUser.getId(),key);
	    } catch (FinderException fe) {
		throw new HeroException(fe.getMessage());
	    } 	
	    
	    if (!c.isEmpty()){	    
		
		propertyLocal=(BnUserPropertyLocal)(c.toArray())[0];
		propertyLocal.setTheValue(value);
		
	    }else{
		try{
		    propertyLocal=pHome.create(key,value);
		    propertyLocal.setBnUser(mUser);
		    
		} catch(InvalidValueException ie){
		    throw new HeroException(ie.getMessage()); 	
		} catch (javax.ejb.CreateException ce) {
		    throw new HeroException(ce.getMessage());
		}     	    
	    }      
/*
	    hero.interfaces.UserSessionLocalHome usersessionhome = (UserSessionLocalHome)hero.interfaces.UserSessionUtil.getLocalHome();
	    UserSessionLocal user = usersessionhome.create();
	    user.setUserProperty(key,value);*/

	} catch (Exception f) {
	    throw new HeroException("Error in setUserProperty");
	}
	log.debug("setUserProperty - end");
    }

    
    public void setSessionContext(javax.ejb.SessionContext context) throws RemoteException {
	this.mContext=context;
	try {
	    userhome = BnUserUtil.getLocalHome();
	    rolehome = BnAuthRoleUtil.getLocalHome();
	}catch(Exception e){
	    throw new RemoteException(e.getMessage());
	}
    }

    public void unsetSessionContext() throws RemoteException {
    }
    
    public void ejbRemove() throws java.rmi.RemoteException {
    }
    
    public void ejbActivate() throws java.rmi.RemoteException {
    }
    
    public void ejbPassivate() throws java.rmi.RemoteException {
    }

}
