package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* UserSessionEJB.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.BnUserPropertyLocalHome;
import hero.interfaces.BnUserPropertyUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.EngineLocal;
import hero.interfaces.EngineLocalHome;
import hero.interfaces.EngineUtil;
import hero.interfaces.InvalidValueException;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.util.EngineException;
import hero.util.HeroException;
import hero.util.JMSEvents;
import hero.util.EventConstants;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Category;

/**
 * Session Bean Template
 *
 * @ejb:bean name="UserSession"
 *           display-name="UserSession Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/UserSession"
 *           local-jndi-name="ejb/hero/UserSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnUser"
 *              ref-name="myhero/BnUser"
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="Admin,user"
 *
 * @jonas.bean
 *      ejb-name="UserSession"
 *      jndi-name="ejb/hero/UserSession"
 **/

public class UserSessionBean implements SessionBean, EventConstants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log =
		Category.getInstance(UserSessionBean.class);

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private BnUserLocalHome userhome;
	private BnUserLocal mUser;
	private EngineLocalHome enghome;
	private EngineLocal mEngine;
	private BnProjectLocalHome projhome;
	private BnNodeLocalHome nodehome;
	private ProjectSessionHome projectseshome;
	private JMSEvents jms;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------

	/**
	 * @return the list of projects of the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectList() throws HeroException {
		log.debug("getProjectList");
		Collection projects = null;
		log.debug("before getprojects mUser " + mUser);
		projects = mUser.getBnProjects();
		BnUserValue uv = mUser.getBnUserValue();
		log.debug("BnUser Value=" + uv);
		hero.interfaces.BnProjectLightValue[] pv = uv.getBnProjects();
		log.debug("BnProject Value " + pv);
		Collection pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
		log.debug("Projects " + pva);
		return pva;
	}

	/**
	 * @delete a Workflow project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void removeProject(String projectName) throws HeroException {
		log.debug("removeProject projectName=" + projectName);
		BnProjectLocalHome pHome;
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			BnProjectLocal pl = pHome.findByName(projectName);
			String userName = mContext.getCallerPrincipal().getName();
			if (userName.equals(pl.getCreator()))
			{
				pl.remove();
				jms.sendProjectEvent(DELETEPROJECT, projectName);
			}
			else
				throw new HeroException("Error in delete project: only the project creator can removed it !!");
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException("BnUser does not exist");
		} catch (RemoveException re) {
			throw new HeroException("Error in delete project");
	}

	}

	/**
	 * @return the names of projects of the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectListNames() throws HeroException {
		log.debug("getProjectListNames");
		Collection projects = null;
		log.debug("before getprojects mUser " + mUser);
		projects = mUser.getBnProjects();
		BnUserValue uv = mUser.getBnUserValue();
		log.debug("BnUser Value=" + uv);
		hero.interfaces.BnProjectLightValue[] pv = uv.getBnProjects();
		log.debug("BnProject Value " + pv);
		Collection pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
		Collection projectsNames = new ArrayList();
		Iterator i = pva.iterator();
		while (i.hasNext())
			projectsNames.add(((hero.interfaces.BnProjectLightValue) i.next()).getName());

		return projectsNames;
	}

	/**
	* Method that obtain all users Activities from specific project (ready
	* state)
	*/
	/**
	 * @return the list of todo activities ot the user for a project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getToDoList(String projectName) throws HeroException {
		log.debug("getToDoList projectName=" + projectName);
		BnProjectLocal project;
		Collection ready;
		try {
			projhome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
				"BnProject " + projectName + " does not exists");
		}
		log.debug("getToDoList -end projectName=" + projectName);
		try {
			ready = nodehome.findUserReady(project.getId(), mUser.getName());
		} catch (FinderException fe) {
			log.debug("Empty Todo List for BnUser " + mUser.getName());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			log.debug("NODE NAME TODO " + nd.getName());
			result.add(nd.getName());
		}
		return result;
	}

	/**
	 * Method that obtain all users Activities from specific project (executingi
	 * and anticipating state)
	 */
	/**
	 * @return the list of active activities ot the user for a project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getActivityList(String projectName)
		throws HeroException {
		BnProjectLocal project;
		Collection executing;
		log.debug("getActivityList -begin projectName=" + projectName);
		try {
			projhome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
				"BnProject " + projectName + " does not exists");
		}
		try {
			executing =
				nodehome.findUserExecuting(project.getId(), mUser.getName());
		} catch (FinderException fe) {
			log.debug("Empty Activity List for BnUser " + mUser.getName());
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			log.debug("NODE NAME Executing " + nd.getName());
			result.add(nd.getName());
		}
		return result;
	}

	//     public String getXMLUserState() throws RemoteException, HeroException {
	// // 	BnProject project;
	// // 	String result="<userstate name=\""+this.getName()+"\">";
	// // 	Collection projects=this.getProjectList();
	// // 	for (Iterator i=projects.iterator();i.hasNext();) {
	// // 	    String projectName=(String)i.next();
	// // 	    result=result+"<project name=\""+projectName+"\">";
	// // 	    try {
	// // 		project = projecthome.findByPrimaryKey(projectName);
	// // 	    } catch (FinderException f) {
	// // 		throw new HeroException("it's not normal"+f.getMessage());
	// // 	    }

	// // 	    Collection todo=project.getUserReadyActivities(this.getName());
	// // 	    if (!todo.isEmpty()) {
	// // 		result=result+"<todo>";
	// // 		for (Iterator j=todo.iterator();j.hasNext();) {
	// // 		    String node=(String)j.next();
	// // 		    BnNode nd=project.getNode(node);
	// // 		    result=result+"<node name=\""+node+"\" state=\""+nd.getState()+"\"/>";
	// // 		}
	// // 		result=result+"</todo>";
	// // 	    }
	// // 	    Collection act = project.getUserActiveActivities(this.getName());
	// // 	    if (!act.isEmpty()) {
	// // 		result=result+"<executing>";
	// // 		for (Iterator k=act.iterator();k.hasNext();) {
	// // 		    String node=(String)k.next();
	// // 		    BnNode nd=project.getNode(node);
	// // 		    result=result+"<node name=\""+node+"\" state=\""+nd.getState()+"\"/>";
	// // 		}
	// // 		result=result+"</executing>";
	// // 	    }
	// // 	    result=result+"</project>";
	// // 	}

	// // 	result=result+"</userstate>";

	// // 	return result;
	//     }

	//     public Collection getUserRoles() throws HeroException {
	// //       log.debug("getUserRoles");
	// // 	try {
	// // 	    return((Vector)user.getRoles());
	// // 	}catch(RemoteException e){
	// // 	    throw new HeroException("Cannot terminate activity");
	// // 	}
	//     }

	/**
	 * tries to terminate a project
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void terminate(String projectName) throws HeroException {
		log.debug("terminate projectName=" + projectName);
		try {
			EngineLocal engine = enghome.create();
			engine.initProject(projectName);
			engine.terminate();
		} catch (CreateException c) {
			throw new EJBException(
				"Cannot create Engine bean for " + projectName);
		} catch (EngineException e) {
			throw new HeroException(
				"Cannot terminate project " + e.getMessage());
		}
		log.debug("terminate");
	}

	/**
	 * tries to start an activity
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/

	public void startActivity(String projectName, String nodeName)
		throws HeroException {
		log.debug(
			"startActivity projectName="
				+ projectName
				+ " nodeName="
				+ nodeName);
		try {
			EngineLocal engine = enghome.create();
			engine.initProject(projectName);
			engine.startActivity(nodeName);
		} catch (CreateException c) {
			throw new HeroException(
				"Cannot create Engine bean for "
					+ projectName
					+ " "
					+ c.getMessage());
		}
		log.debug(
			"startActivity -end projectName="
				+ projectName
				+ " nodeName="
				+ nodeName);
	}

	/**
	 * tries to terminate an activity
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void terminateActivity(String projectName, String nodeName)
		throws HeroException {
		log.debug(
			"terminateActivity projectName="
				+ projectName
				+ " nodeName="
				+ nodeName);
		try {
			EngineLocal engine = enghome.create();
			engine.initProject(projectName);
			engine.terminateActivity(nodeName);
		} catch (CreateException c) {
			throw new HeroException(
				"Cannot create Engine bean for " + projectName);
		}
		log.debug(
			"terminateActivity -end projectName="
				+ projectName
				+ " nodeName="
				+ nodeName);
	}

	/**
	 * tries to cancel an activity
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void cancelActivity(String projectName, String nodeName)
		throws HeroException {
		log.debug(
			"cancelActivity projectName="
				+ projectName
				+ " nodeName="
				+ nodeName);
		try {
			EngineLocal engine = enghome.create();
			engine.initProject(projectName);
			engine.cancelActivity(nodeName);
		} catch (CreateException c) {
			throw new EJBException(
				"Cannot create Engine bean for " + projectName);
		}
		log.debug(
			"cancelActivity -end projectName="
				+ projectName
				+ " nodeName="
				+ nodeName);
	}

	/**
	 * @return a BnNodeValue for the project AND the node
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public BnNodeValue getNode(String projectName, String nodeName)
		throws HeroException {
		log.debug(
			"getNode projectName=" + projectName + " nodeName=" + nodeName);
		try {
			ProjectSessionLocalHome projectseshome =
				ProjectSessionUtil.getLocalHome();
			ProjectSessionLocal project = projectseshome.create();
			project.initProject(projectName);
			if (!checkRights(project))
				throw new HeroException("BnUser is not member of the project");
			return project.getNodeValue(nodeName);
		} catch (CreateException f) {
			throw new HeroException("BnProject does not exists " + projectName);
		} catch (javax.naming.NamingException r) {
			throw new HeroException(
				"Cannot access to project " + r.getMessage());
		}
	}

	/**
	 * @param projectName the name of the project
	 * @param roleName the new role for the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void addRole(String roleName) throws HeroException {
		log.debug("addRole roleName=" + roleName);
		throw new HeroException("Not yes implemented");
	}

	/**
	 * returns the user name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUser() {
		return mUser.getName();
	}

	/**
	 * returns the user name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public BnUserLightValue getUserLightValue() {
		return mUser.getBnUserLightValue();
	}

	/**
	 * returns the user password
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUserPassword() {
		return mUser.getPassword();
	}

	/**
	  * Get BnUser properties o
	  *
	  * @return A BnUserPropertyValue collection of all user properties
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Supports"
	  *
	  **/
	public Collection getUserProperties() throws HeroException {
		Collection prp = new ArrayList();
		BnUserPropertyLocal pl;

		Collection pls = mUser.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnUserPropertyLocal) i.next();
			prp.add(pl.getBnUserPropertyValue());
		}
		return (prp);
	}

	/**
	 * Set a property of a user
	 *
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/

	public void setUserProperty(String key, String value)
		throws HeroException {
		BnUserPropertyLocalHome pHome;
		BnUserPropertyLocal propertyLocal = null;

		log.debug("setUserProperty  key=" + key + " value=" + value);

		try {
			pHome = BnUserPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mUser.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			propertyLocal = (BnUserPropertyLocal) (c.toArray())[0];
			propertyLocal.setTheValue(value);

		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnUser(mUser);

			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {
				throw new HeroException(ce.getMessage());
			}

		}

	}

	/**
	  * Delete a property 
	  *
	  * @param key the key of the property
	  * @ejb:interface-method view-type="both"
	  * @ejb:transaction type="Required"
	  *
	  **/
	public void deleteUserProperty(String key) throws HeroException {
		log.debug("deleteUserProperty key=" + key);

		BnUserPropertyLocalHome pHome;
		BnUserPropertyLocal propertyLocal = null;

		try {
			pHome = BnUserPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mUser.getId(), key);
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				propertyLocal = (BnUserPropertyLocal) (c.toArray())[0];
				propertyLocal.remove();
			} catch (RemoveException rm) {
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
	}

	public void setNodeProperty(
		String project,
		String node,
		String key,
		String value)
		throws HeroException {
		// 	log.debug("setNodeProperty project="+project+" node="+node+" key="+key+" value="+value);
		// 	try {
		// 	    BnProject prj=projecthome.findByPrimaryKey(project);
		// 	    prj.setNodeProperty(node,key,value);
		// 	} catch (RemoteException r) {
		// 	    throw new HeroException("Cannot access project "+r.getMessage());
		// 	} catch (FinderException f) {
		// 	    throw new HeroException("BnProject does not exists "+f.getMessage());
		// 	}
		// 	log.debug("setNodeProperty -end project="+project+" node="+node+" key="+key+" value="+value);
	}

	//     public String getNodeProperty(String project, String node, String key) throws HeroException {
	// // 	log.debug("getNodeProperty project="+project+" node="+node+" key="+key);
	// // 	try {
	// // 	    BnProject prj=projecthome.findByPrimaryKey(project);
	// // 	    String prop=prj.getNodeProperty(node,key);
	// // 	    log.debug("getNodeProperty -end project="+project+" node="+node+" key="+key);
	// // 	    return prop;
	// // 	} catch (RemoteException r) {
	// // 	    throw new HeroException("Cannot access project "+r.getMessage());
	// // 	} catch (FinderException f) {
	// // 	    throw new HeroException("BnProject does not exists "+f.getMessage());
	// // 	}
	//     }

	/**
	* Create the BnUser Session Bean
	*
	* @throws CreateException 
	*
	* @ejb:create-method view-type="both"
	**/
	public void ejbCreate() throws CreateException {
		String userName = null;
		log.debug("ejbCreate BnUser");
		try {
			userName = mContext.getCallerPrincipal().getName();
			log.debug("session create usrname=" + userName);
			userhome = hero.interfaces.BnUserUtil.getLocalHome();
			enghome = EngineUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
			mUser = userhome.findByName(userName);
		} catch (javax.naming.NamingException ne) {
			throw new CreateException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new CreateException(userName + " does not exists");
		} catch (Exception e) {
			log.debug("ejbCreate failed. Try init method");
		}
	}

	/**
	 * Init the BnUser Session Bean
	 *
	 * @throws CreateException 
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void init() throws CreateException {
		String userName = null;
		try {
			if (mUser == null) {
				userName = mContext.getCallerPrincipal().getName();
				log.debug("session create usrname=" + userName);
				userhome = hero.interfaces.BnUserUtil.getLocalHome();
				enghome = EngineUtil.getLocalHome();
				nodehome = BnNodeUtil.getLocalHome();
				mUser = userhome.findByName(userName);
			}
		} catch (javax.naming.NamingException ne) {
			throw new CreateException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new CreateException(userName + " does not exists");
		}

	}

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		jms = new JMSEvents();
	}

	public void ejbRemove() {
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

	// utility methods
	private boolean checkRights(ProjectSessionLocal ps) throws HeroException {
		// check that current user is a member of the project
		return ps.containsUser(mContext.getCallerPrincipal().getName());
	}
}
