/*
*
* JMSServicesException.java - 
* Copyright (C) 2002 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package hero.util;

import javax.jms.*;
import javax.naming.*;
import java.io.*;
import java.util.*;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.jonas.service.Service;


public class JMSServices {

    private static JMSServices jms = null; // Singleton pattern

    String                  topicName = "testTopic";
    Context                 jndiContext = null;
    TopicConnectionFactory  topicConnectionFactory = null;
    TopicSession            topicSession = null;
    Topic                   topic = null;
    TopicConnection         topicConnection = null;
    TopicSubscriber         topicSubscriber = null;
    TopicPublisher          topicPublisher = null;
    TextMessage             message = null;
    ObjectMessage           omessage = null;
    InputStreamReader       inputStreamReader = null;

    private JMSServices() throws Exception{             

        /* 
         * Create a JNDI InitialContext object if none exists yet.
         */

        try {
            jndiContext = new InitialContext();
        } catch (NamingException e) {
            System.out.println("Could not create JNDI " +
                "context: " + e.toString());
            System.exit(1);
        }

        /*
         * Look up connection factory and topic.  If either does
         * not exist, exit.
         */


        try {
            //topicConnectionFactory = (TopicConnectionFactory)
            //         jndiContext.lookup("UILXAConnectionFactory");
           
			try{
//			  if a ServiceException is caught, it seems that we are called in a client
			Service sjms = ServiceManager.getJmsService();
//			  here we know we are called by a bean
			topicConnectionFactory = (TopicConnectionFactory) jndiContext.lookup("TCF");
			}catch (ServiceException se){
			topicConnectionFactory = (TopicConnectionFactory) jndiContext.lookup("JTCF");
			}

			topic = (Topic) jndiContext.lookup(topicName);            
        } catch (NamingException e) {
            System.out.println("JNDI lookup failed: " 
                + e.toString());
            System.exit(1);
        }

	/*
	 * Create connection.
	 * Create session from connection; false means session is
	 * not transacted.
	 */
	 

	try { 
	    topicConnection = topicConnectionFactory.createTopicConnection();
	    topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);   
	} catch (JMSException e) {
	    System.out.println("Exception occurred: " + e.toString());
	}

    }
    
    /**
     * Returns the instance of the JMSServices
     * Implementation of the Singleton pattern.
    */
    
    public static JMSServices getInstance() throws Exception {
        if (jms == null) {
           jms = new JMSServices();
        }   
        return jms;

    }

    /**
     * Create subscriber.
     * Register message listener (TextListener).
     * Receive text messages from topic.
     */ 

    public void createSubscription(MessageListener listener, String evt) throws Exception {

	try {
	    topicSubscriber = topicSession.createSubscriber(topic, evt, false);
	    topicSubscriber.setMessageListener(listener);
	    topicConnection.start();
	} catch (JMSException e) {
	  System.out.println("Exception occurred: " 
			     + e.toString());
	}
    }


    /**
     * Create publisher and text message.
     * Send messages, varying text slightly.
     * Finally, close connection.
     */
    
    public void send(Hashtable evt, String type) throws Exception{

	StringBuffer subs = new StringBuffer("");

        try {
            topicPublisher = topicSession.createPublisher(topic);
            message = topicSession.createTextMessage();

	    Enumeration keys = evt.keys();
	    while (keys.hasMoreElements())
	    {
	      String key = (String)keys.nextElement();
	      String value = (String)evt.get(key);
       	      message.setStringProperty(key, value);
	      subs.append(" "+key+" = '"+value +"' AND ");
	      evt.put(key,value);
	    }
	    message.setJMSType(type);

	    String res = subs.toString();
	    String result = res.substring(0,res.length()-4);
	    message.setText(result);
		      
	    topicPublisher.publish(message);
	    
		topicSession.close();
     	topicConnection.close();
		jms=null;

        } catch (JMSException e) {
            System.out.println("Exception occurred: " + 
                e.toString());
	}
    }

    public void send(java.io.Serializable o, String type) throws Exception{

	StringBuffer subs = new StringBuffer("");

        try {
            topicPublisher = topicSession.createPublisher(topic);
            omessage = topicSession.createObjectMessage();
	    omessage.setJMSType(type);
	    omessage.setObject(o);
		      
	    topicPublisher.publish(omessage);
		topicSession.close();
 	    topicConnection.close();
		jms=null;


        } catch (JMSException e) {
            System.out.println("Exception occurred: " + 
                e.toString());
	}
    }

    public static Hashtable getEvt(Message  message) throws JMSException
    {
	Enumeration keys = message.getPropertyNames();
	Hashtable evt = new Hashtable();
	
	while (keys.hasMoreElements())
	{
	    String key = (String)keys.nextElement();
	    evt.put(key,message.getObjectProperty(key));
	}
	return evt;
    }

    public static java.io.Serializable getEvtObject(Message  message) throws JMSException
    {
	return (((ObjectMessage)message).getObject());
    }
    
    public void closeConnection() {
	
	if (topicConnection != null) {
	    try {
		topicConnection.close();
	    } catch (JMSException e) {}
	}
    }
	
}
